@@ -17,8 +17,9 @@ import uuid
 from oslo_config import cfg
 from oslo_log import log as logging
 import oslo_messaging as messaging
-from oslo_messaging.notify import dispatcher as oslo_dispatcher
+from oslo_messaging.rpc import dispatcher
 from oslo_messaging import target
+from oslo_service import service
 from oslo_utils import timeutils
 import pytz
 from sqlalchemy import desc
@@ -28,14 +29,10 @@ from murano.db import models
 from murano.db.services import environments
 from murano.db.services import instances
 from murano.db import session
-from murano.common.i18n import _LI, _LW
 from murano.services import states
 
 CONF = cfg.CONF
 
-RPC_SERVICE = None
-NOTIFICATION_SERVICE = None
-
 LOG = logging.getLogger(__name__)
 
 
@@ -53,8 +50,8 @@ class ResultEndpoint(object):
         environment = unit.query(models.Environment).get(environment_id)
 
         if not environment:
-            LOG.warning(_LW('Environment result could not be handled, '
-                            'specified environment not found in database'))
+            LOG.warning('Environment result could not be handled, '
+                        'specified environment not found in database')
             return
 
         if model['Objects'] is None and model.get('ObjectsCopy', {}) is None:
@@ -116,14 +113,14 @@ class ResultEndpoint(object):
         if objects:
             services = objects.get('services')
         if num_errors + num_warnings > 0:
-            LOG.warning(_LW('EnvId: {env_id} TenantId: {tenant_id} Status: '
-                            'Failed Apps: {services}')
+            LOG.warning('EnvId: {env_id} TenantId: {tenant_id} Status: '
+                        'Failed Apps: {services}'
                         .format(env_id=environment.id,
                                 tenant_id=environment.tenant_id,
                                 services=services))
         else:
-            LOG.info(_LI('EnvId: {env_id} TenantId: {tenant_id} Status: '
-                         'Successful Apps: {services}')
+            LOG.info('EnvId: {env_id} TenantId: {tenant_id} Status: '
+                     'Successful Apps: {services}'
                      .format(env_id=environment.id,
                              tenant_id=environment.tenant_id,
                              services=services))
@@ -177,8 +174,7 @@ def report_notification(report):
     LOG.debug('Got report from orchestration '
               'engine:\n{report}'.format(report=report))
 
-    report['entity_id'] = report['id']
-    del report['id']
+    report['entity_id'] = report.pop('id')
 
     status = models.Status()
     if 'timestamp' in report:
@@ -202,35 +198,78 @@ def get_last_deployment(unit, env_id):
     return query.first()
 
 
-def _prepare_rpc_service(server_id):
-    endpoints = [ResultEndpoint()]
+class Service(service.Service):
+    """Service class, that contains common methods for custom services"""
 
-    transport = messaging.get_transport(CONF)
-    s_target = target.Target('murano', 'results', server=server_id)
-    return messaging.get_rpc_server(transport, s_target, endpoints, 'eventlet')
+    def __init__(self):
+        super(Service, self).__init__()
+        self.server = None
+
+    def stop(self, graceful=False):
+        if self.server:
+            self.server.stop()
+            if graceful:
+                self.server.wait()
+        super(Service, self).stop()
+
+    def reset(self):
+        if self.server:
+            self.server.reset()
+        super(Service, self).reset()
 
 
-def _prepare_notification_service(server_id):
+def get_notification_listener():
+
     endpoints = [report_notification, track_instance, untrack_instance]
+    transport = messaging.get_notification_transport(CONF)
+    s_target = target.Target(topic='murano', server=str(uuid.uuid4()))
+    listener = messaging.get_notification_listener(
+        transport, [s_target], endpoints, executor='threading')
 
+    return listener
+
+
+def get_rpc_server():
+
+    endpoints = [ResultEndpoint()]
     transport = messaging.get_transport(CONF)
-    s_target = target.Target(topic='murano', server=server_id)
-    dispatcher = oslo_dispatcher.NotificationDispatcher(
-        [s_target], endpoints, None, True)
-    return messaging.MessageHandlingServer(transport, dispatcher, 'eventlet')
+    s_target = target.Target('murano', 'results', server=str(uuid.uuid4()))
+    access_policy = dispatcher.DefaultRPCAccessPolicy
+    server = messaging.get_rpc_server(
+        transport, s_target, endpoints, 'threading',
+        access_policy=access_policy)
+
+    return server
 
 
-def get_rpc_service():
-    global RPC_SERVICE
+class NotificationService(Service):
+    def __init__(self):
+        super(NotificationService, self).__init__()
+        self.server = None
 
-    if RPC_SERVICE is None:
-        RPC_SERVICE = _prepare_rpc_service(str(uuid.uuid4()))
-    return RPC_SERVICE
+    def start(self):
+        endpoints = [report_notification, track_instance, untrack_instance]
+
+        transport = messaging.get_notification_transport(CONF)
+        s_target = target.Target(topic='murano', server=str(uuid.uuid4()))
+
+        self.server = messaging.get_notification_listener(
+            transport, [s_target], endpoints, executor='eventlet')
+
+        self.server.start()
+        super(NotificationService, self).start()
 
 
-def get_notification_service():
-    global NOTIFICATION_SERVICE
+class ApiService(Service):
 
-    if NOTIFICATION_SERVICE is None:
-        NOTIFICATION_SERVICE = _prepare_notification_service(str(uuid.uuid4()))
-    return NOTIFICATION_SERVICE
+    def start(self):
+        endpoints = [ResultEndpoint()]
+
+        transport = messaging.get_transport(CONF)
+        s_target = target.Target('murano', 'results', server=str(uuid.uuid4()))
+        access_policy = dispatcher.DefaultRPCAccessPolicy
+        self.server = messaging.get_rpc_server(
+            transport, s_target, endpoints, 'eventlet',
+            access_policy=access_policy)
+        self.server.start()
+        super(ApiService, self).start()
